'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var core = require('@react2rx/core');
var _JSON$stringify = require('@babel/runtime-corejs3/core-js/json/stringify');
var _concatInstanceProperty = require('@babel/runtime-corejs3/core-js/instance/concat');
var _forEachInstanceProperty = require('@babel/runtime-corejs3/core-js/instance/for-each');
var lodash = require('@rontrol/lodash');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var _JSON$stringify__default = /*#__PURE__*/_interopDefaultLegacy(_JSON$stringify);
var _concatInstanceProperty__default = /*#__PURE__*/_interopDefaultLegacy(_concatInstanceProperty);
var _forEachInstanceProperty__default = /*#__PURE__*/_interopDefaultLegacy(_forEachInstanceProperty);

var formatActorStage = actor => {
  var _context, _context2;
  var opts = lodash.some(actor.opts, (_, k) => k !== "parentActor") ? ":".concat( /*#__PURE__*/_JSON$stringify__default["default"](actor.opts)) : "";
  return [/*#__PURE__*/_concatInstanceProperty__default["default"](_context = /*#__PURE__*/_concatInstanceProperty__default["default"](_context2 = /*#__PURE__*/"@@".concat(actor.group, "/")).call(_context2, actor.name)).call(_context, opts), actor.stage];
};
var printBuffer = (buffer, _ref) => {
  var {
    styles,
    titleFormatter,
    actionTransformer,
    level
  } = _ref;
  var print = (entry, index) => {
    var {
      started,
      startedTime,
      action,
      prevState,
      error
    } = entry;
    var {
      took,
      nextState
    } = entry;
    var nextEntry = buffer[index + 1];
    if (nextEntry) {
      nextState = nextEntry.prevState;
      took = nextEntry.started - started;
    }
    var formattedAction = /*#__PURE__*/actionTransformer(action);
    var titleStr = /*#__PURE__*/formatActorStage(formattedAction);
    var title = /*#__PURE__*/titleFormatter(titleStr, startedTime, took);

    // eslint-disable-next-line
    console.groupCollapsed( /*#__PURE__*/"%c ".concat(title), ...styles.title(titleStr, startedTime, took));
    var makeLevel = () => {
      if (typeof level === "string") {
        return {
          prevStateLevel: level,
          actionLevel: level,
          nextStateLevel: level,
          errorLevel: level
        };
      }
      return {
        prevStateLevel: /*#__PURE__*/level["prevState"](prevState),
        actionLevel: /*#__PURE__*/level["action"](action),
        nextStateLevel: /*#__PURE__*/level["nextState"](nextState),
        errorLevel: /*#__PURE__*/level["error"](error, prevState)
      };
    };
    var {
      prevStateLevel,
      actionLevel,
      nextStateLevel,
      errorLevel
    } = /*#__PURE__*/makeLevel();

    // eslint-disable-next-line
    console[prevStateLevel]("%c prev state ", /*#__PURE__*/styles.prevState(prevState), prevState);

    // eslint-disable-next-line
    console[actionLevel]("%c action     ", /*#__PURE__*/styles.action(formattedAction), formattedAction);
    if (error) {
      // eslint-disable-next-line
      console[errorLevel]("%c error      ", /*#__PURE__*/styles.error(error, prevState), error);
    }

    // eslint-disable-next-line
    console[nextStateLevel]("%c next state ", /*#__PURE__*/styles.nextState(nextState), nextState);

    // eslint-disable-next-line
    console.groupEnd();
  };
  _forEachInstanceProperty__default["default"](buffer).call(buffer, print);
};

var timer = typeof performance !== "undefined" && performance !== null && typeof performance.now === "function" ? performance : Date;
var repeat = (str, times) => new Array(times + 1).join(str);
var pad = (num, maxLength) => {
  var _context;
  return _concatInstanceProperty__default["default"](_context = /*#__PURE__*/"".concat( /*#__PURE__*/repeat("0", maxLength - num.toString().length))).call(_context, num);
};
var formatTime = time => {
  var _context2, _context3, _context4;
  return _concatInstanceProperty__default["default"](_context2 = /*#__PURE__*/_concatInstanceProperty__default["default"](_context3 = /*#__PURE__*/_concatInstanceProperty__default["default"](_context4 = /*#__PURE__*/"".concat( /*#__PURE__*/pad( /*#__PURE__*/time.getHours(), 2), ":")).call(_context4, /*#__PURE__*/pad( /*#__PURE__*/time.getMinutes(), 2), ":")).call(_context3, /*#__PURE__*/pad( /*#__PURE__*/time.getSeconds(), 2), ".")).call(_context2, /*#__PURE__*/pad( /*#__PURE__*/time.getMilliseconds(), 3));
};

var stageStyles = {
  STARTED: "color:#0958d9;",
  DONE: "color:#389e0d;",
  FAILED: "color:#c41d7f;",
  CANCEL: "color:#d46b08;"
};
var defOptions = {
  styles: {
    title: (_ref, time, took) => {
      var [_, stage] = _ref;
      var isDark = window.matchMedia("(prefers-color-scheme: dark)").matches;
      var parts = ["color:#f5f5f5;background:#0958d9;padding:2px 6px 2px 6px;border-radius:0 10px 10px 0;", /*#__PURE__*/"color:".concat(isDark ? "#d6e4ff" : "#030852", ";font-size:12px;margin:0 2px 0 2px;")];
      stage && parts.push(stageStyles[stage]);
      time && parts.push("color:#999;");
      took && parts.push("color:#999;");
      return parts;
    },
    prevState: () => "color:#f5f5f5;background:#d4b106;padding:2px;border-radius:2px;font-weight: bold;",
    action: () => "color:#f5f5f5;background:#0958d9;padding:2px;border-radius:2px;font-weight: bold;",
    nextState: () => "color:#f5f5f5;background:#cf1322;padding:2px;border-radius:2px;font-weight: bold;",
    error: () => "color:#f5f5f5;background:#d4380d;padding:2px;border-radius:2px;font-weight: bold;"
  },
  level: "log",
  stateTransformer: state => state,
  actionTransformer: action => action,
  errorTransformer: error => error,
  titleFormatter(_ref2, time, took) {
    var [title, stage] = _ref2;
    var parts = ["action", /*#__PURE__*/"%c ".concat(title)];
    stage && parts.push( /*#__PURE__*/"%c".concat(stage));
    time && parts.push( /*#__PURE__*/"%c@ ".concat(time));
    took && parts.push( /*#__PURE__*/"%c(in ".concat( /*#__PURE__*/took.toFixed(2), " ms)"));
    return parts.join(" ");
  }
};
var logEntry = (_ref3, action) => {
  var {
    getState
  } = _ref3;
  var started = /*#__PURE__*/timer.now();
  var startedTime = /*#__PURE__*/formatTime(new Date());
  return {
    started,
    startedTime,
    action,
    prevState: /*#__PURE__*/getState(),
    nextState: {},
    took: 0
  };
};
var createLogger = options => api => next => {
  var opts = /*#__PURE__*/core.recorder(options, defOptions);
  var logBuffer = [];
  return action => {
    var entry = /*#__PURE__*/logEntry(api, action);
    logBuffer.push(entry);
    var returnedValue;
    try {
      returnedValue = /*#__PURE__*/next(action);
    } catch (e) {
      entry.error = e;
      returnedValue = /*#__PURE__*/core.actor.doWith(action, e);
    }
    entry.took = timer.now() - entry.started;
    entry.nextState = /*#__PURE__*/api.getState();
    printBuffer(logBuffer, opts);
    logBuffer.length = 0;
    if (entry.error) {
      throw entry.error;
    }
    return returnedValue;
  };
};

exports.createLogger = createLogger;
